package com.sf.payment.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.sf.common.utils.SecurityUtils;
import com.sf.order.domain.OrderInfo;
import com.sf.order.service.IOrderInfoService;
import com.sf.payment.config.HuaweiPaymentConfig;
import com.sf.payment.domain.HuaweiPurchasesVerifyDTO;
import com.sf.payment.domain.HuaweiPurchasesVerifyResponseDTO;
import com.sf.payment.domain.InAppPurchaseData;
import com.sf.payment.service.IHuaweiPaymentService;
import com.sf.system.domain.UserMember;
import com.sf.system.service.IUserMemberService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.Security;
import java.security.spec.X509EncodedKeySpec;
import java.text.MessageFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


/**
 * 功能描述:
 *
 * @author a_kun
 * @date 2024/4/12 10:21
 */
@Slf4j
@Service
public class HuaweiPaymentServiceImpl implements IHuaweiPaymentService {

    // token url to get the authorization
    private static final String TOKEN_URL = "https://oauth-login.cloud.huawei.com/oauth2/v3/token";
    private static final String VERIFY_TOKEN_URL = "https://orders-drcn.iap.cloud.huawei.com.cn/applications/purchases/tokens/verify";

    private static final String PUBLIC_KEY = "PUBLIC_KEY";

    @Autowired
    private IOrderInfoService orderInfoService;

    @Autowired
    private IUserMemberService userMemberService;

    @Autowired
    private HuaweiPaymentConfig huaweiPaymentConfig;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void purchasesVerify(HuaweiPurchasesVerifyDTO verifyDTO) {
        // construct the Authorization in Header
        Map<String, String> headers = buildAuthorization(getAppAT(huaweiPaymentConfig.getClientId(), huaweiPaymentConfig.getClientSecret()));

        // pack the request body
        Map<String, String> bodyMap = new HashMap<>();
        bodyMap.put("purchaseToken", verifyDTO.getPurchaseToken());
        bodyMap.put("productId", verifyDTO.getProductId());

        String response = HttpUtil.createPost(VERIFY_TOKEN_URL)
                .addHeaders(headers)
                .body(JSON.toJSONString(bodyMap))
                .execute().body();
        HuaweiPurchasesVerifyResponseDTO huaweiPurchasesVerifyResponseDTO = JSON.parseObject(response, HuaweiPurchasesVerifyResponseDTO.class);
        InAppPurchaseData inAppPurchaseData = JSON.parseObject(huaweiPurchasesVerifyResponseDTO.getPurchaseTokenData(), InAppPurchaseData.class);
        // 获取服务订单
        OrderInfo orderInfo = orderInfoService.selectOrderInfoByOrderNo(verifyDTO.getOrderNo());
        // 校验订单
        boolean checkSuccessOrder = checkSuccessOrder(huaweiPurchasesVerifyResponseDTO.getPurchaseTokenData()
                , huaweiPurchasesVerifyResponseDTO.getDataSignature()
                , PUBLIC_KEY
                , huaweiPurchasesVerifyResponseDTO.getSignatureAlgorithm()
                , orderInfo);

        Assert.isTrue(checkSuccessOrder, "订单校验失败，请重试");
        Assert.isTrue(inAppPurchaseData.getPurchaseState()==0, "订单未完成购买");
        DateTime payTime = DateUtil.date(inAppPurchaseData.getPurchaseTime());
        UserMember userMember = userMemberService.selectUserMemberByUserId(orderInfo.getCreateUserId());
        if (userMember == null) {
            // 添加会员信息
            boolean isSubscription = verifyDTO.getKind() == 2;
            userMember = new UserMember();
            userMember.setMemberLevel(isSubscription ? 1 : 2);
            userMember.setSubscriptionStatus(isSubscription ? 1 : 0);
            userMember.setUserId(orderInfo.getCreateUserId());
            userMember.setIntegration(0L);
            DateTime expirationTime = DateUtil.offset(payTime, DateField.MONTH, 1);
            userMember.setExpirationTime(expirationTime);
            userMember.setCreateTime(new Date());
            userMember.setUpdateTime(new Date());
            userMemberService.insertUserMember(userMember);
        } else {
            // 更新
            DateTime expirationTime = DateUtil.offset(userMember.getExpirationTime(), DateField.MONTH, 1);
            userMember.setExpirationTime(expirationTime);
            userMember.setUpdateTime(new Date());
            userMemberService.updateUserMember(userMember);
        }
        // 更新订单状态
        orderInfo.setPayTime(payTime);
        orderInfo.setOrderStatus(3L);
        orderInfo.setPayChannel(2L);
        orderInfo.setPayAmt(orderInfo.getOrderAmt());
        orderInfo.setReallyAmt(orderInfo.getOrderAmt());
        orderInfo.setOrderStatus(3L);
        orderInfo.setPayData(huaweiPurchasesVerifyResponseDTO.getPurchaseTokenData());
        orderInfoService.updateOrderInfo(orderInfo);
    }

    /**
     * Gets App Level AccessToken.
     */
    public static String getAppAT(String clientId, String clientSecret) {
        // fetch accessToken
        Map<String, Object> form = new HashMap<>(8);
        form.put("grant_type", "client_credentials");
        form.put("client_secret", clientSecret);
        form.put("client_id", clientId);
        String atResponse = HttpUtil.post(TOKEN_URL, form);
        log.info("getAppAT Response : {}", atResponse);
        JSONObject parseObject = JSON.parseObject(atResponse);
        return parseObject.getString("access_token");
    }

    /**
     * Build Authorization in Header
     *
     * @param appAt appAt
     * @return headers
     */
    public static Map<String, String> buildAuthorization(String appAt) {
        String oriString = MessageFormat.format("APPAT:{0}", appAt);
        String authorization =
                MessageFormat.format("Basic {0}", Base64.encodeBase64String(oriString.getBytes(StandardCharsets.UTF_8)));
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", authorization);
        headers.put("Content-Type", "application/json; charset=UTF-8");
        return headers;
    }

    /**
     * 校验签名信息，校验InAppPurchaseData中的productId、price、currency等信息的一致性
     *
     * @param content   结果字符串
     * @param sign      签名字符串
     * @param publicKey IAP公钥
     * @return 是否校验通过
     */
    public static boolean checkSuccessOrder(String content, String sign, String publicKey, String signatureAlgorithm, OrderInfo orderInfo) {
        // 校验签名信息
        boolean checkRes = checkSign(content, sign, publicKey, StrUtil.blankToDefault(signatureAlgorithm, "SHA256WithRSA"));
        if (checkRes) {
            // 校验InAppPurchaseData中的productId、price、currency等信息的一致性
            checkRes = checkProductIdAndPriceAndCurrency(content, orderInfo);
        }
        return checkRes;
    }

    /**
     * 校验InAppPurchaseData中的productId、price、currency等信息的一致性
     *
     * @param content       结果字符串
     * @param yourOrderInfo 您的订单信息，包括productId、price、currency
     * @return 是否校验通过
     */
    public static boolean checkProductIdAndPriceAndCurrency(String content, OrderInfo yourOrderInfo) {
        InAppPurchaseData inAppPurchaseData = JSON.parseObject(content, InAppPurchaseData.class);
        // 校验InAppPurchaseData中的productId、price、currency等信息的一致性
        return inAppPurchaseData.getProductId().equals(yourOrderInfo.getGoodsId())
                && inAppPurchaseData.getPrice().equals(yourOrderInfo.getOrderAmt());
    }

    /**
     * 校验签名信息
     *
     * @param content            结果字符串
     * @param sign               签名字符串
     * @param publicKey          IAP公钥
     * @param signatureAlgorithm 签名算法字段，可从接口返回数据中获取，例如：OwnedPurchasesResult.getSignatureAlgorithm()
     * @return 是否校验通过
     */
    public static boolean checkSign(String content, String sign, String publicKey, String signatureAlgorithm) {
        if (sign == null) {
            return false;
        }
        if (publicKey == null) {
            return false;
        }

        // 当signatureAlgorithm为空时使用默认签名算法
        if (signatureAlgorithm == null || signatureAlgorithm.length() == 0) {
            signatureAlgorithm = "SHA256WithRSA";
            System.out.println("doCheck, algorithm: SHA256WithRSA");
        }
        try {
            Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
            // 生成"RSA"的KeyFactory对象
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            byte[] decodedKey = Base64.decodeBase64(publicKey);
            // 生成公钥
            PublicKey pubKey = keyFactory.generatePublic(new X509EncodedKeySpec(decodedKey));
            java.security.Signature signature = null;
            // 根据SHA256WithRSA算法获取签名对象实例
            signature = java.security.Signature.getInstance(signatureAlgorithm);
            // 初始化验证签名的公钥
            signature.initVerify(pubKey);
            // 把原始报文更新到签名对象中
            signature.update(content.getBytes(StandardCharsets.UTF_8));
            // 将sign解码
            byte[] bsign = Base64.decodeBase64(sign);
            // 进行验签
            return signature.verify(bsign);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

}
